#include <stdio.h>
#include <avr/io.h>

#include "SCP1000-D01.h"

#define sbi(var, mask)   ((var) |= (uint8_t)(1 << mask))
#define cbi(var, mask)   ((var) &= (uint8_t)~(1 << mask))

//General short delays
//Uses internal timer do a fairly accurate 1us
void delay_us(uint16_t x)
{
/*
	while(x > 256)
	{
		TIFR0 = (1<<TOV0); //Clear any interrupt flags on Timer2
		TCNT0 = 0; //256 - 125 = 131 : Preload timer 2 for x clicks. Should be 1us per click
		while( (TIFR0 & (1<<TOV0)) == 0);
		
		x -= 256;
	}

	TIFR0 = (1<<TOV0); //Clear any interrupt flags on Timer2
	TCNT0 = 256 - x; //256 - 125 = 131 : Preload timer 2 for x clicks. Should be 1us per click
	while( (TIFR0 & (1<<TOV0)) == 0);
*/

	for(volatile int z = 0; z<500; z++){}
}

//General short delays
void delay_ms(uint16_t x)
{
	for ( ; x > 0 ; x--)
		delay_us(1000);
}

char scp_init(void)
{
	int8_t temp, x, i;

	// TODO
    DDRB = 0b11101111; //PB4 = MISO 
    DDRC = 0b11111010; //PC0 = DRDY, PC2 = SCP_MISO, PC4 = SDA, PC5 = SCL
    DDRD = 0b11011110; //PORTD (RX on PD0) PD5 = MISO


	//Poll SCP until it is ready
/*
	while(1)
	{
		sbi(PORTC, CSB); // De-Select SPI Device
		delay_ms(10);
		temp = SCP_read_register(0x00);
		temp = SCP_read_register(0x00);
		
		temp = SCP_read_register(0x07); //Check STATUS for LSB = 0
		if ( !(temp & 0x01) ) break;

		printf("\n\nPolling SCP1000 (1) ... temp = %02x", temp);
	}
*/
	
	// RESET
	SCP_write_register(0x06, 0x01);   
	delay_ms(50);
	
 	for(i=1; i<7; i++) 
 	{
   		x = SCP_read_register(0x07);         //STATUS register
   		printf("STATUS(%u) = %u\n",i,x);      //LSB should be 0 if startup OK
   		if ((x&1)==0) break;
   			delay_ms(10);
   }
   x = SCP_read_register(0x1F);   //DATARD8 register
   printf("DATARD8 = %u\n",x);      //should equal 1 if eeprom checksum OK

    //Configure SCP1000 with low noise configuration
    //=====================================
	SCP_write_register(0x02, 0x2D); //Write indirect register address to 0x02
    SCP_write_register(0x01, 0x03); //Write 0x03 to indirect register data
    SCP_write_register(0x03, 0x02); //Write this data to the indirect register
    delay_ms(50);
	
    SCP_write_register(0x03, 0x00); //Reset operation mode
    delay_ms(50);

    SCP_write_register(0x03, 0x0A); //Select High Resolution Mode
    delay_ms(50);
}



char scp_read(float *temperature_o, double *pressure_o)
{
    uint8_t temp_data;
    unsigned int scp_temperature;
    unsigned long int pressure_scp;

	//Wait for new data
	while(1)
	{
		temp_data = SCP_read_register(0x07); //Check the status register for bit 5 to go high
		if ( temp_data & 0b00100000 ) 
			break;

		if ( temp_data & 0b00010000 ) 
			return -1;
		
		if ( PINC & (1 << DRDY) ) 
			break;
	}


	scp_temperature = SCP_read_register16(0x21); //Read the temperature data
	if(scp_temperature > 8192) //Then this is a negative temp
	{
		scp_temperature ^= 0x1FFF; //Convert 2's compliment
		scp_temperature &= 0x1FFF;
		scp_temperature += 1;
		scp_temperature *= -1;
	}

	*temperature_o = scp_temperature / 20.0;
	
	pressure_scp = SCP_read_register(0x1F) & 0x0007; //Read MSB pressure data - 3 lower bits
	pressure_scp <<= 16; //Shift it
	pressure_scp |= SCP_read_register16(0x20);
	
	*pressure_o = pressure_scp / 4.0;
	
	return 0;
}	

//Read 8-bit register
uint8_t SCP_read_register(uint8_t register_name)
{
    uint8_t in_byte;
    
    register_name <<= 2;
    register_name &= 0b11111100; //Read command

    cbi(PORTC, CSB); //Select SPI Device

    in_byte = SCP_spi_comm(register_name); //Write byte to device
    //in_byte is nothing, we need to clock in another 8 bits
    in_byte = SCP_spi_comm(0x00); //Send nothing, but we should get back the register value

    sbi(PORTC, CSB);
    
    return(in_byte);
}

//Read 16-bit register
uint16_t SCP_read_register16(uint8_t register_name)
{
    uint16_t in_byte;

    register_name <<= 2;
    register_name &= 0b11111100; //Read command

    cbi(PORTC, CSB); //Select SPI Device

    in_byte = SCP_spi_comm(register_name); //Write byte to device
    //in_byte is nothing, we need to clock in another 8 bits
    in_byte = SCP_spi_comm(0x00); //Send nothing, but we should get back the register value
    in_byte <<= 8;
    in_byte |= SCP_spi_comm(0x00); //Send nothing, but we should get back the register value

    sbi(PORTC, CSB);
    
    return(in_byte);
}

//Sends a write command to SCP1000
void SCP_write_register(uint8_t register_name, uint8_t register_value)
{
    uint8_t in_byte;
    
    register_name <<= 2;
    register_name |= 0b00000010; //Write command

    cbi(PORTC, CSB); //Select SPI device

    in_byte = SCP_spi_comm(register_name); //Send register location
    in_byte = SCP_spi_comm(register_value); //Send value to record into register
    
    sbi(PORTC, CSB);
    
    //Return nothing
}

//Basic SPI send and receive
uint8_t SCP_spi_comm(uint8_t outgoing_byte)
{
    uint8_t incoming_byte, x;

    incoming_byte = 0;
    
    for(x = 8 ; x > 0 ; x--)
    {
        cbi(PORTB, SCK); //Toggle the SPI clock
		delay_ms(1);

        //Put bit on SPI data bus    
        if(outgoing_byte & (1 << (x-1)))
            sbi(PORTC, SCP_MOSI);
        else
            cbi(PORTC, SCP_MOSI);

		delay_ms(1);
        sbi(PORTB, SCK);

        //Read bit on SPI data bus
        incoming_byte <<= 1;
        if ( (PINC & (1 << SCP_MISO)) ) incoming_byte |= 1;
		delay_ms(1);
    }
    
    return(incoming_byte);
}
